using skyscraper5.Mpeg2;
using skyscraper5.src.InteractionChannel.Model;
using skyscraper5.src.InteractionChannel.Model.Descriptors;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Net.NetworkInformation;
using System.Text;
using System.Threading.Tasks;
using skyscraper5.src.InteractionChannel.Model2;
using skyscraper8.InteractionChannel.Model;
using skyscraper8.InteractionChannel.Model2;
using skyscraper8.InteractionChannel.Model2.Descriptors;

namespace skyscraper5.src.InteractionChannel
{
	internal class NullInteractionChannelHandler : InteractionChannelHandler
	{
		public int GetRmtTransmissionStandard(ushort networkId)
		{
			return 0;
		}

		public void OnBroadcastConfiguration(ushort networkId, Bct bct)
		{
		}

		public void OnCorrectionMessage(ushort interactiveNetworkId, Cmt cmt)
		{
		}

		public void OnFrameComposition(ushort interactiveNetworkId, Fct fct)
		{
		}

		public void OnFrameComposition2(ushort networkId, Fct2 fct2)
		{
		}

		public void OnInteractionChannelError(InteractionChannelErrorState unexpectedTable)
		{
		}

		public void OnRcs2Nit(ushort interactiveNetworkId, RcsNit nit)
		{
		}

		public void OnRcs2Tdt(ushort interactiveNetworkId, Rcs2Tdt tdt)
		{
		}

		public void OnRcsMap(Rmt rmt)
		{
		}

		public void OnSatellitePosition(ushort interactiveNetworkId, Spt spt)
		{
		}

		public void OnSuperframeComposition(ushort interactiveNetworkId, Sct sct)
		{
		}

		public void OnTerminalBurstTimePlan(ushort interactiveNetworkId, Tbtp tbtp)
		{
		}

		public void OnTerminalBurstTimePlan2(ushort interactiveNetworkId, Tbtp2 tbtp2)
		{
		}

		public void OnTerminalInformationMessage(ushort? interactiveNetworkId, PhysicalAddress physicalAddress, Tim tim)
		{
		}

		public void OnTimeslotComposition(ushort interactiveNetworkId, Tct tct)
		{
		}

		public void OnTransmissionModeSupport(ushort interactiveNetworkId, Tmst tmst)
		{
		}

		public void OnTransmissionModeSupport2(ushort interactiveNetworkId, Tmst2 tmst2)
		{
		}
	}
}
